#!/usr/bin/env python3

"""
Create cookie molds from DXF files.

(c) 2023, Matthew Petroff <https://mpetroff.net/>
License: CC0 1.0, https://creativecommons.org/publicdomain/zero/1.0/
"""

import glob
import numpy as np
import cadquery as cq

# Dimensions in millimeters

# Cookie size
height = 53
width = 43
dough_thickness = 6
pattern_thickness = 1

base_thickness = dough_thickness - pattern_thickness

# Fillet radius of edge of pattern
r_fillet = 4.5

# If we make the cookie base the exact size of the pattern, CadQuery produces
# incorrect geometry; thus, we enlarge it slightly
edge_margin = 0.1

# Cookie mold extra size
extra_size = 2

# Apply a draft angle to the base of the cookie only
# CadQuery can't handle doing so automatically for the pattern, since it has
# interior lines
draft_angle = 10  # deg


def create_cookie_mold(name):
    # Calculate enlargement due to draft angle of cookie base
    draft_diff = np.sin(np.deg2rad(draft_angle)) * base_thickness

    # Create starting block
    result = cq.Workplane("XY").box(
        width + extra_size + draft_diff * 2 + edge_margin * 2,
        height + extra_size + draft_diff * 2 + edge_margin * 2,
        base_thickness + pattern_thickness + extra_size,
    )
    result = result.edges("|Z").fillet(
        r_fillet + extra_size / 2 + draft_diff + edge_margin
    )
    block_volume = result.val().Volume()
    result = result.faces(">Z").workplane()

    # Cut away cookie base
    base = (
        cq.Sketch()
        .rect(
            width + draft_diff * 2 + edge_margin * 2,
            height + draft_diff * 2 + edge_margin * 2,
        )
        .vertices()
        .fillet(r_fillet + draft_diff + edge_margin)
    )
    result = result.placeSketch(base).cutBlind(-base_thickness, taper=draft_angle)

    # Cut away cookie pattern
    pattern = cq.Sketch()
    for f in sorted(glob.glob(name + "_*.dxf")):
        # This doesn't work for nested shapes, so they're in different files
        # (I think it will work if they're in different layers, but Inkscape
        #  doesn't export them like that)
        pattern = pattern.importDXF(f)
    result = result.center(-width / 2, -height / 2)
    result = result.placeSketch(pattern).cutBlind(-base_thickness - pattern_thickness)

    # Cut away most of edge, so dough comes out of mold
    result = result.faces(">Z").workplane()
    result = result.center(width / 2, height / 2)
    base = cq.Sketch().rect(width * 2, height * 0.75)
    result = result.placeSketch(base).cutBlind(
        -base_thickness * 0.75, taper=draft_angle * 2
    )
    base = cq.Sketch().rect(width * 0.75, height * 2)
    result = result.faces(">Z").workplane()
    result = result.placeSketch(base).cutBlind(
        -base_thickness * 0.75, taper=draft_angle * 2
    )

    # Mirror, so cookies are correct
    result = result.mirror("YZ")
    mold_volume = result.val().Volume()

    # Export STEP file
    cq.exporters.export(result, name + ".step")


for i in range(1, 13):
    create_cookie_mold(f"cookie{i:02d}")
